#!/usr/bin/env python
# -*- coding: utf-8  -*-

# rename.py, script che rinomina files.
# Copyright (C) 2007  Roberto Zanon
#
# This program is free software: you can redistribute it and/or modify it under 
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Script che rinomina secondo un certo pattern i file contenuti in una directory.

Sintassi: python rename.py [opzioni] espressDa espressA [directory]
	espessDa: l'espressione regolare che seleziona i file da rinominare.
	espessA: l'espressione che definisce il nuovo nome.
	directory: la directory in cui si trovano i file da rinominare;
		se non e` specificata e` usata la directory corrente.
	
	Opzioni:	
	  -r, --rename-only :
	 	rinomina (senza chiedere conferma).
	  -s, --show-only :
		visualizza quello che verrebbe rinominato senza questa opzione
		(senza fare nessun tipo di modifica).
	  Senza nessuna opzione chiede conferma ed, eventualmente, rinomina.
		
Puo` anche essere usato come libereria python invece che come script:
	importando rename e usando la funzione rename.rename(...)

"""

import sys, os, re
from optparse import OptionParser

VERSION = "0.1"

class flags():
	"""Definisce le modalità in cui puo` funzionare rename.
	"""
	RINOMINA_CHIEDI=1
	RINOMINA_SOLO=2
	VISUALIZZA_SOLO=3

def rename(espressDa, espressA, directory, flag=flags.RINOMINA_CHIEDI):
	"""Rinomina i file contenuti in "directory" secondo le espressioni.
	
	se flag vale RINOMINA_SOLO, rinomina senza chiedere conferma.
	se flag vale VISUALIZZA_SOLO, visualizza cosa verrebbe rinominato.
	Se flag vale RINOMINA_CHIEDI o non è specificato,
		rinomina i file dopo aver chiesto conferma.

	Solleva eccezioni se la directory o le espressioni non sono valide. 
	"""
	directory += os.sep
	
	regex = re.compile(espressDa)
	for nomeFile in os.listdir(directory):
		newNomefile = regex.sub(espressA, nomeFile)
		if nomeFile != newNomefile:
			if not flag == flags.RINOMINA_SOLO:
				print '"' + nomeFile + '" -> "' + newNomefile + '"'
				if os.path.exists(directory + newNomefile):
					print "WARNING: '" + newNomefile + "' esiste."
			if flag == flags.RINOMINA_CHIEDI:
				risp = raw_input("Rinomino? ([Si]|No): ")
				rispSi = risp in ["S", "s", "Si", ""]
			if flag == flags.RINOMINA_SOLO or \
					(flag == flags.RINOMINA_CHIEDI and rispSi):
				os.rename(directory+nomeFile, directory+newNomefile)


def main():
	"""Esguito quando rename.py e` chiamato come script.
	
	Parsa gli argomenti, e chiama rename(...).
	"""
	uso = " %prog [opzioni] espressDa espressA [directory]\n" + \
			"\t%prog --help, per l'help completo."
	parser = OptionParser(usage=uso, version="%prog " + VERSION)
	flag = flags.RINOMINA_CHIEDI
	parser.add_option("-r", "--rename-only", action="store_const", \
						const=flags.RINOMINA_SOLO, dest="flag", \
						help="rinomina (senza chiedere conferma)")
	parser.add_option("-s", "--show-only", action="store_const", \
						const=flags.VISUALIZZA_SOLO, dest="flag", \
						help="visualizza quello che sarebbe rinominato senza questa opzione")
	(opzioni, argomenti) = parser.parse_args()
	if len(argomenti) > 3 or len(argomenti) < 2:
		parser.error("Numero di argomenti non valido")
	
	espressDa = argomenti[0]
	espressA = argomenti[1]
	directory = argomenti[2] if len(argomenti) == 3 else os.getcwd()
	
	try:
		rename(espressDa, espressA, directory, flag)
	except (IOError, OSError), e:
		print >> sys.stderr, e
	except Exception, e:
		print >> sys.stderr, "Errore nell'espressione regolare: ", e

if __name__ == "__main__":
	main()
